==> draive-0.62.2.tar.gz: draive-0.62.2/src/draive/resources/types.py <==
from collections.abc import Sequence
from typing import Any, Protocol, Self, overload, runtime_checkable

from haiway import Default, State

from draive.commons import META_EMPTY, Meta
from draive.multimodal import (
    MediaData,
    MediaReference,
    MultimodalContentElement,
    TextContent,
    # validated_media_type,
)
from draive.parameters import DataModel

__all__ = (
    "Resource",
    "ResourceContent",
    "ResourceDeclaration",
    "ResourceException",
    "ResourceFetching",
    "ResourceListFetching",
    "ResourceMissing",
)


class ResourceException(Exception):
    pass


class ResourceMissing(ResourceException):
    pass


class ResourceDeclaration(DataModel):
    uri: str
    mime_type: str | None
    name: str
    description: str | None = None
    meta: Meta = Default(META_EMPTY)


# TODO: add ResourceTemplateDeclaration for ResourceTemplates support
# https://modelcontextprotocol.io/docs/concepts/resources#resource-templates


class ResourceContent(State):
    @overload
    @classmethod
    def of(
        cls,
        content: bytes,
        /,
        *,
        mime_type: str,
    ) -> Self: ...

    @overload
    @classmethod
    def of(
        cls,
        content: MultimodalContentElement | str,
        /,
        *,
        mime_type: str | None = None,
    ) -> Self: ...

    @classmethod
    def of(
        cls,
        content: MultimodalContentElement | str | bytes,
        /,
        *,
        mime_type: str | None = None,
    ) -> Self:
        match content:
            case str() as text:
                return cls(
                    mime_type=mime_type or "text/plain",
                    blob=text.encode(),
                )

            case bytes() as data:
                return cls(
                    mime_type=mime_type or "application/octet-stream",
                    blob=data,
                )

            case TextContent() as text_content:
                return cls(
                    mime_type=mime_type or "text/plain",
                    blob=text_content.text.encode(),
                )

            case MediaData() as media_data:
                return cls(
                    mime_type=mime_type or media_data.media,
                    blob=media_data.data,
                )

            case MediaReference():
                raise ValueError("Resource can't use a media reference")

            case other:
                return cls(
                    mime_type=mime_type or "application/json",
                    blob=other.as_json().encode(),
                )

        return cls()

    mime_type: str
    blob: bytes

    def as_multimodal(self) -> MultimodalContentElement:
        match self.mime_type:
            case "text/plain":
                return TextContent(text=self.blob.decode())

            case "application/json":
                return DataModel.from_json(self.blob.decode())

            case other:
                # try to match supported media
                return MediaData.of(
                    self.blob,
                    media=other,
                )


class Resource(State):
    uri: str
    name: str
    description: str | None
    content: Sequence[Self] | ResourceContent
    meta: Meta = Default(META_EMPTY)


@runtime_checkable
class ResourceFetching(Protocol):
    async def __call__(
        self,
        uri: str,
        **extra: Any,
    ) -> Resource | None: ...


@runtime_checkable
class ResourceListFetching(Protocol):
    async def __call__(
        self,
        **extra: Any,
    ) -> Sequence[ResourceDeclaration]: ...

==> draive-0.63.1.tar.gz: draive-0.63.1/src/draive/resources/types.py <==
from collections.abc import Sequence
from typing import Any, Protocol, Self, overload, runtime_checkable

from haiway import Default, State

from draive.commons import META_EMPTY, Meta
from draive.multimodal import (
    MediaData,
    MediaReference,
    MultimodalContentElement,
    TextContent,
    # validated_media_type,
)
from draive.parameters import DataModel

__all__ = (
    "Resource",
    "ResourceContent",
    "ResourceDeclaration",
    "ResourceException",
    "ResourceFetching",
    "ResourceListFetching",
    "ResourceMissing",
)


class ResourceException(Exception):
    pass


class ResourceMissing(ResourceException):
    pass


class ResourceDeclaration(DataModel):
    uri: str
    mime_type: str | None
    name: str
    description: str | None = None
    meta: Meta = Default(META_EMPTY)


# TODO: add ResourceTemplateDeclaration for ResourceTemplates support
# https://modelcontextprotocol.io/docs/concepts/resources#resource-templates


class ResourceContent(State):
    @overload
    @classmethod
    def of(
        cls,
        content: bytes,
        /,
        *,
        mime_type: str,
    ) -> Self: ...

    @overload
    @classmethod
    def of(
        cls,
        content: MultimodalContentElement | str,
        /,
        *,
        mime_type: str | None = None,
    ) -> Self: ...

    @classmethod
    def of(
        cls,
        content: MultimodalContentElement | str | bytes,
        /,
        *,
        mime_type: str | None = None,
    ) -> Self:
        match content:
            case str() as text:
                return cls(
                    mime_type=mime_type or "text/plain",
                    blob=text.encode(),
                )

            case bytes() as data:
                return cls(
                    mime_type=mime_type or "application/octet-stream",
                    blob=data,
                )

            case TextContent() as text_content:
                return cls(
                    mime_type=mime_type or "text/plain",
                    blob=text_content.text.encode(),
                )

            case MediaData() as media_data:
                return cls(
                    mime_type=mime_type or media_data.media,
                    blob=media_data.data,
                )

            case MediaReference():
                raise ValueError("Resource can't use a media reference")

            case other:
                return cls(
                    mime_type=mime_type or "application/json",
                    blob=other.to_json().encode(),
                )

        return cls()

    mime_type: str
    blob: bytes

    def to_multimodal(self) -> MultimodalContentElement:
        match self.mime_type:
            case "text/plain":
                return TextContent(text=self.blob.decode())

            case "application/json":
                return DataModel.from_json(self.blob.decode())

            case other:
                # try to match supported media
                return MediaData.of(
                    self.blob,
                    media=other,
                )


class Resource(State):
    uri: str
    name: str
    description: str | None
    content: Sequence[Self] | ResourceContent
    meta: Meta = Default(META_EMPTY)


@runtime_checkable
class ResourceFetching(Protocol):
    async def __call__(
        self,
        uri: str,
        **extra: Any,
    ) -> Resource | None: ...


@runtime_checkable
class ResourceListFetching(Protocol):
    async def __call__(
        self,
        **extra: Any,
    ) -> Sequence[ResourceDeclaration]: ...
